#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>

typedef struct
{
    char titul[51];
    char autor[51];
    unsigned rokVydani;
    enum{CD,DVD,BR,VHS} typ;
}TMedium;

typedef enum{KONEC=0,NOVA,PRIDAT,VYHLEDAT,SMAZAT,EXPORT,IMPORT,OPEN} TAkce;

TAkce menu();
void nactiRetezec(char *string,char * label,int maxSize);
FILE *vytvorSoubor(char *fileName);
TMedium nactiMedium();
void zapisMedium(FILE *f,TMedium data);
int najdiMedium(char *what,FILE *where,TMedium *found);
void vypisMedium(FILE *where,TMedium what);
int smaz(char *what,FILE *where);
void exportData(FILE *from,FILE *to);
void importData(FILE *to, FILE *from);
int removeNL(char *string);
void clearBuffer();

int main()
{
    TAkce akce;
    FILE *f;
    char fileName[256];
    TMedium medium;
    int res;
    char hledanyTitul[51];

    while((akce=menu())!=KONEC)
    {
        switch(akce)
        {
            case NOVA:nactiRetezec(fileName,"Jmeno souboru: ",255);
                      f = vytvorSoubor(fileName);
                      break;
            case PRIDAT:
                      medium = nactiMedium();
                      zapisMedium(f,medium);
                      break;
            case VYHLEDAT:
                      nactiRetezec(hledanyTitul,"Hledany titul: ",50);
                      TMedium vysledek;
                      res = najdiMedium(hledanyTitul,f,&vysledek);
                      if(res!=0)
                      {
                          vypisMedium(stdout,vysledek);
                      }
                      else
                      {
                          fprintf(stderr,"Medium %s nenalezeno!\n",hledanyTitul);
                      }
                      break;
            case SMAZAT:
                      nactiRetezec(hledanyTitul,"Mazany titul: ",50);
                      res = smaz(hledanyTitul,f);
                      if(res!=0)
                      {
                          printf("Medium %s smazano.\n",hledanyTitul);
                      }
                      else
                      {
                          fprintf(stderr,"Medium %s v kartotece neni.\n",hledanyTitul);
                      }
                      break;
            case EXPORT:
                      nactiRetezec(fileName,"Soubor pro export: ",255);
                      FILE *exp = fopen(fileName,"wt");
                      if(exp==NULL)
                      {
                          fprintf(stderr,"Nelze otevrit soubor pro export.\n");
                          break;
                      }
                      exportData(f,exp);
                      fclose(exp);
                      break;
            case IMPORT:
                      nactiRetezec(fileName,"Soubor pro import: ",255);
                      FILE *inp = fopen(fileName,"rt");
                      if(inp==NULL)
                      {
                          fprintf(stderr,"Soubor %s neexistuje nebo ho nelze otevrit.\n",fileName);
                          break;
                      }
                      importData(f,inp);
                      fclose(inp);
                      break;
            case OPEN:nactiRetezec(fileName,"Soubor s daty: ",255);
                      f = fopen(fileName,"r+b");
                      if(!f)
                      {
                          f = fopen(fileName,"w+b");
                      }
                      break;
            default:;
        }
    }
    return 0;
}

TAkce menu()
{
    int value=-INT_MAX;
    printf("1. vytvorit novou kartoteku\n");
    printf("2. pridat medium\n");
    printf("3. vyhledat medium\n");
    printf("4. smazat medium\n");
    printf("5. exportovat data do CSV\n");
    printf("6. importovat data z CSV\n");
    printf("7. otevrit existujici kartoteku.\n");
    printf("0. konec\n");
    do{
        if(value!=-INT_MAX)
        {
            printf("Chyba zadani. Akce %d neni v nabidce.\n");
        }
        printf("Zadejte vybranou akci [0-7]:");
        scanf("%d",&value);
    }while(value<KONEC || value >IMPORT);
    return value;

}

void nactiRetezec(char *string,char * label,int maxSize)
{
    printf("%s",label);
    fgets(string,maxSize,stdin);
    if(!removeNL(string))
    {
        clearBuffer();
    }
}
int removeNL(char *string)
{
    char *nl = strchr(string,'\n');
    if(nl!=NULL)
    {
        *nl='\0';
        return 1;
    }
    return 0;
}

void clearBuffer()
{
    while(getchar()!='\n');
}

FILE *vytvorSoubor(char *fileName)
{
    return fopen(fileName,"wb");
}
TMedium nactiMedium()
{
    TMedium data;
    nactiRetezec(data.titul,"Nazev media: ",50);
    nactiRetezec(data.autor,"Autor nebo interpret: ",50);
    printf("Rok vydani: ");
    scanf("%u",&(data.rokVydani));
    printf("Typ media (1=CD, 2=DVD, 3=BR, 4=VHS):");
    unsigned typ;
    scanf("%u",&typ);
    data.typ = typ;
    return data;
}
void zapisMedium(FILE *f,TMedium data)
{
    long curpos = ftell(f);
    fseek(f,0,SEEK_END);
    fwrite(&data,sizeof(TMedium),1,f);
    fflush(f);
    fseek(f,curpos,SEEK_SET);
}

int najdiMedium(char *what,FILE *where,TMedium *found)
{
    long curPos = ftell(where);
    rewind(where);
    while(!feof(where))
    {
        fread(found,sizeof(TMedium),1,where);
        if(strcmp(found->titul,what)==0)
        {
            return 1;
        }
    }
    fseek(where,curPos,SEEK_SET);
    return 0;

}
void vypisMedium(FILE *output,TMedium what)
{
    fprintf(output,"%s;%s;%u;",what.titul,what.autor,what.rokVydani);
    switch(what.typ)
    {
        case CD: fprintf(output,"CD\n");
                 break;
        case DVD:fprintf(output,"DVD\n");
                 break;
        case BR: fprintf(output,"BluRay\n");
                 break;
        case VHS:fprintf(output,"VHS\n");
                 break;
    }
}
int smaz(char *what,FILE *where)
{
    return 0;
}
void exportData(FILE *from,FILE *to)
{
    long pos = ftell(from);
    rewind(from);
    while(!feof(from))
    {
        TMedium data;
        fread(&data, sizeof(TMedium),1,from);
        if(!feof(from))
        {
            zapisMedium(to,data);
        }
    }
    fseek(from,pos,SEEK_SET);
}
void importData(FILE *to, FILE *from)
{

}

